#include "metafile.h"

CMetaFile::CMetaFile() : m_bMetaFileOpened(FALSE)
{
}

CMetaFile::~CMetaFile()
{
	if (m_bMetaFileOpened)
        m_oMetaFile.close();
}

bool CMetaFile::OpenMetaFileHandle(const char * strFileName)
{
	/* Close previously opened file, if not closed */
	CloseMetaFileHandle();

	/* open(create) output file */
	m_oMetaFile.open(strFileName, ios::out|ios::trunc);
	if (m_oMetaFile.ios::fail()) 
		return false;	// Error occured when open file

	/* Housekeeping */
	m_bMetaFileOpened = TRUE;

	return true;
}

void CMetaFile::CloseMetaFileHandle()
{
	if (m_bMetaFileOpened) {
		m_oMetaFile.close();
		m_bMetaFileOpened=FALSE;
	}
}

void CMetaFile::WriteToMetaFile(char* buf,int No)
{
    m_oMetaFile.write(buf,No);
}

inline char CODINGTYPECHAR(VOPpredType x) {
    if (IVOP == x) 
        return 'I'; 
    else if (PVOP == x) 
        return 'P'; 
    else if (SPRITE == x)
        return 'S';
    else if (BVOP == x) 
        return 'B'; 
    else 
        return 'x'; 
}


// write Vop meta data
void CMetaFile::WriteVopMetaData(int VopNo, UInt file_pos, VOPpredType pretype)
{
    char buf[256];
    // new version
    sprintf(buf, "VOP[%d] = 0x%x, %c", VopNo, file_pos,CODINGTYPECHAR(pretype));
    // old version
    //sprintf(buf, "VOP[%d] = 0x%x", m_ptFile->m_sScalableInfo.m_iVopNo, (uint32)m_ptFile->m_sScalableInfo.m_bm/*m_ptFile->GetCurrentOffset()*/);
    WriteToMetaFile(buf,strlen(buf));
}

void CMetaFile::WriteVopMetaData2(CStatistics& cStatVOP)
{
    char buf[256];
    // new version
    sprintf(buf, ", Size(%d), IDCT(%d), MC(%d/%d/%d/%d), VLC(%d), PSNR(%.2f,%.2f,%.2f)\n", 
                                                      cStatVOP.nBitsTotal/8,
                                                      0,//m_cStatVOP.nCountDCT,
                                                      cStatVOP.nIntraMB,
                                                      cStatVOP.nForwardMB,
                                                      cStatVOP.nBackwardMB,
                                                      cStatVOP.nInterpolateMB,
                                                      0,//m_cStatVOP.nCountVLC,
                                                      cStatVOP.dSNRY,
                                                      cStatVOP.dSNRU,
                                                      cStatVOP.dSNRV
                                                      );
    // old version
    /*sprintf(buf, ", IDCT(%d), MC(%d/%d/%d/%d), VLC(%d), PSNR(%.2f,%.2f,%.2f)\n",                                                       
                                                      m_ptFile->m_cStatVOP.nCountDCT,
                                                      m_ptFile->m_cStatVOP.nIntraMB,
                                                      m_ptFile->m_cStatVOP.nForwardMB,
                                                      m_ptFile->m_cStatVOP.nBackwardMB,
                                                      m_ptFile->m_cStatVOP.nInterpolateMB,
                                                      m_ptFile->m_cStatVOP.nCountVLC,
                                                      m_ptFile->m_cStatVOP.dSNRY,
                                                      m_ptFile->m_cStatVOP.dSNRU,
                                                      m_ptFile->m_cStatVOP.dSNRV
                                                      );*/
    WriteToMetaFile(buf,strlen(buf));
}

void CMetaFile::WriteEOSMetaData(UInt file_pos)
{
    char buf[256];
    sprintf(buf, "VOP.EOS = 0x%x\n", file_pos);
    WriteToMetaFile(buf,strlen(buf));
}

void CMetaFile::WriteFGSVopMetaData(int iVopNo, int maxlevel, int codedlevel)
{
    char buf[256];
    sprintf(buf, "VOP[%d].MaxLevel = %d, CodedLevel = %d\n", iVopNo, maxlevel, codedlevel);
    WriteToMetaFile(buf,strlen(buf));
}

void CMetaFile::WriteFGSVopBpMetaData(int iVopNo, int fgsVopBpID, UInt file_pos)
{
    char buf[256];
    sprintf(buf, "VOP[%d].BP[%d] = 0x%x", iVopNo, fgsVopBpID, file_pos);
    WriteToMetaFile(buf,strlen(buf));
}